package sun.security.jgss.krb5;

import sun.security.jgss.spi.GSSCredentialSpi;
import java.security.PrivilegedActionException;
import java.security.AccessControlContext;
import java.security.PrivilegedExceptionAction;
import java.security.AccessController;
import javax.security.auth.DestroyFailedException;
import java.security.Provider;
import org.ietf.jgss.Oid;
import sun.security.jgss.spi.GSSNameSpi;
import sun.security.krb5.PrincipalName;
import sun.security.krb5.EncryptionKey;
import sun.security.jgss.GSSCaller;
import java.io.IOException;
import sun.security.krb5.KrbException;
import org.ietf.jgss.GSSException;
import sun.security.krb5.KerberosSecrets;
import java.net.InetAddress;
import java.util.Date;
import javax.security.auth.kerberos.KerberosPrincipal;
import sun.security.krb5.Credentials;
import javax.security.auth.kerberos.KerberosTicket;

public class Krb5InitCredential extends KerberosTicket implements Krb5CredElement
{
    private static final long serialVersionUID = 7723415700837898232L;
    private Krb5NameElement name;
    private Credentials krb5Credentials;
    public KerberosTicket proxyTicket;
    
    private Krb5InitCredential(Krb5NameElement name, byte[] array, KerberosPrincipal client, KerberosPrincipal kerberosPrincipal, KerberosPrincipal server, KerberosPrincipal kerberosPrincipal2, byte[] array2, int n, boolean[] array3, Date date, Date date2, Date date3, Date date4, InetAddress[] array4) throws GSSException {
        super(array, client, server, array2, n, array3, date, date2, date3, date4, array4);
        KerberosSecrets.getJavaxSecurityAuthKerberosAccess().kerberosTicketSetClientAlias(this, kerberosPrincipal);
        KerberosSecrets.getJavaxSecurityAuthKerberosAccess().kerberosTicketSetServerAlias(this, kerberosPrincipal2);
        this.name = name;
        try {
            this.krb5Credentials = new Credentials(array, client.getName(), (kerberosPrincipal != null) ? kerberosPrincipal.getName() : null, server.getName(), (kerberosPrincipal2 != null) ? kerberosPrincipal2.getName() : null, array2, n, array3, date, date2, date3, date4, array4);
        }
        catch (KrbException ex) {
            throw new GSSException(13, -1, ex.getMessage());
        }
        catch (IOException ex2) {
            throw new GSSException(13, -1, ex2.getMessage());
        }
    }
    
    private Krb5InitCredential(Krb5NameElement name, Credentials krb5Credentials, byte[] asn1Encoding, KerberosPrincipal client, KerberosPrincipal kerberosPrincipal, KerberosPrincipal server, KerberosPrincipal kerberosPrincipal2, byte[] sessionKey, int keyType, boolean[] flags, Date authTime, Date startTime, Date endTime, Date renewTill, InetAddress[] clientAddresses) throws GSSException {
        super(asn1Encoding, client, server, sessionKey, keyType, flags, authTime, startTime, endTime, renewTill, clientAddresses);
        KerberosSecrets.getJavaxSecurityAuthKerberosAccess().kerberosTicketSetClientAlias(this, kerberosPrincipal);
        KerberosSecrets.getJavaxSecurityAuthKerberosAccess().kerberosTicketSetServerAlias(this, kerberosPrincipal2);
        this.name = name;
        this.krb5Credentials = krb5Credentials;
    }
    
    static Krb5InitCredential getInstance(GSSCaller gssCaller, Krb5NameElement instance, int n) throws GSSException {
        KerberosTicket tgt = getTgt(gssCaller, instance, n);
        if (tgt == null) {
            throw new GSSException(13, -1, "Failed to find any Kerberos tgt");
        }
        if (instance == null) {
            instance = Krb5NameElement.getInstance(tgt.getClient().getName(), Krb5MechFactory.NT_GSS_KRB5_PRINCIPAL);
        }
        Krb5InitCredential o = new Krb5InitCredential(instance, tgt.getEncoded(), tgt.getClient(), KerberosSecrets.getJavaxSecurityAuthKerberosAccess().kerberosTicketGetClientAlias(tgt), tgt.getServer(), KerberosSecrets.getJavaxSecurityAuthKerberosAccess().kerberosTicketGetServerAlias(tgt), tgt.getSessionKey().getEncoded(), tgt.getSessionKeyType(), tgt.getFlags(), tgt.getAuthTime(), tgt.getStartTime(), tgt.getEndTime(), tgt.getRenewTill(), tgt.getClientAddresses());
        o.proxyTicket = KerberosSecrets.getJavaxSecurityAuthKerberosAccess().kerberosTicketGetProxy(tgt);
        return o;
    }
    
    static Krb5InitCredential getInstance(Krb5NameElement krb5NameElement, Credentials credentials) throws GSSException {
        EncryptionKey sessionKey = credentials.getSessionKey();
        PrincipalName client = credentials.getClient();
        PrincipalName clientAlias = credentials.getClientAlias();
        PrincipalName server = credentials.getServer();
        PrincipalName serverAlias = credentials.getServerAlias();
        KerberosPrincipal kerberosPrincipal = null;
        Object o = null;
        Object o2 = null;
        Object o3 = null;
        Krb5NameElement instance = null;
        if (client != null) {
            String name = client.getName();
            instance = Krb5NameElement.getInstance(name, Krb5MechFactory.NT_GSS_KRB5_PRINCIPAL);
            kerberosPrincipal = new KerberosPrincipal(name);
        }
        if (clientAlias != null) {
            o = new KerberosPrincipal(clientAlias.getName());
        }
        if (server != null) {
            o2 = new KerberosPrincipal(server.getName(), 2);
        }
        if (serverAlias != null) {
            o3 = new KerberosPrincipal(serverAlias.getName());
        }
        return new Krb5InitCredential(instance, credentials, credentials.getEncoded(), kerberosPrincipal, o, o2, o3, sessionKey.getBytes(), sessionKey.getEType(), credentials.getFlags(), credentials.getAuthTime(), credentials.getStartTime(), credentials.getEndTime(), credentials.getRenewTill(), credentials.getClientAddresses());
    }
    
    @Override
    public final GSSNameSpi getName() throws GSSException {
        return this.name;
    }
    
    @Override
    public int getInitLifetime() throws GSSException {
        Date endTime = this.getEndTime();
        if (endTime == null) {
            return 0;
        }
        return (int)((endTime.getTime() - System.currentTimeMillis()) / 1000L);
    }
    
    @Override
    public int getAcceptLifetime() throws GSSException {
        return 0;
    }
    
    @Override
    public boolean isInitiatorCredential() throws GSSException {
        return true;
    }
    
    @Override
    public boolean isAcceptorCredential() throws GSSException {
        return false;
    }
    
    @Override
    public final Oid getMechanism() {
        return Krb5MechFactory.GSS_KRB5_MECH_OID;
    }
    
    @Override
    public final Provider getProvider() {
        return Krb5MechFactory.PROVIDER;
    }
    
    Credentials getKrb5Credentials() {
        return this.krb5Credentials;
    }
    
    @Override
    public void dispose() throws GSSException {
        try {
            this.destroy();
        }
        catch (DestroyFailedException cause) {
            new GSSException(11, -1, "Could not destroy credentials - " + cause.getMessage()).initCause(cause);
        }
    }
    
    private static KerberosTicket getTgt(final GSSCaller gssCaller, Krb5NameElement krb5NameElement, int n) throws GSSException {
        final String name;
        if (krb5NameElement != null) {
            name = krb5NameElement.getKrb5PrincipalName().getName();
        }
        else {
            name = null;
        }
        final AccessControlContext context = AccessController.getContext();
        try {
            gssCaller = ((gssCaller == GSSCaller.CALLER_UNKNOWN) ? GSSCaller.CALLER_INITIATE : gssCaller);
            return (KerberosTicket)AccessController.doPrivileged(new PrivilegedExceptionAction<KerberosTicket>() {
                @Override
                public KerberosTicket run() throws Exception {
                    return Krb5Util.getInitialTicket(gssCaller, name, context);
                }
            });
        }
        catch (PrivilegedActionException ex) {
            GSSException o = new GSSException(13, -1, "Attempt to obtain new INITIATE credentials failed! (" + ex.getMessage() + ")");
            o.initCause(ex.getException());
            throw o;
        }
    }
    
    @Override
    public GSSCredentialSpi impersonate(GSSNameSpi gssNameSpi) throws GSSException {
        try {
            Krb5NameElement krb5NameElement = (Krb5NameElement)gssNameSpi;
            return new Krb5ProxyCredential(this, krb5NameElement, Credentials.acquireS4U2selfCreds(krb5NameElement.getKrb5PrincipalName(), this.krb5Credentials).getTicket());
        }
        catch (IOException | KrbException cause) {
            GSSException ex = new GSSException(11, -1, "Attempt to obtain S4U2self credentials failed!");
            ex.initCause(cause);
            throw ex;
        }
    }
}
